#!/usr/bin/env python3
"""
Integration tests for wrapper functionality in the interp submodule.

Public use of this module is not supported by NAG.
"""

# NAG Copyright 2018-2020.

# pylint: disable=invalid-name

import unittest

import numpy as np

from naginterfaces.library import interp as chap, rand
from test_functionality_core import IntegrationTest

class TestE01cf(IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the e01cf wrapper.
    """

class TestE01cfNp(TestE01cf, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        TestE01cf.set_up(self)
        self.interp = chap.dim1_monconv_disc
        self.fun = chap.dim1_monconv_eval

    def test_good(self):
        """
        Good call.
        """
        x = [
            40., 10., 30., 20., 50.,
        ]
        y = [
            6., 3., 4.7, 4., 6.,
        ]
        comm = self.interp(
            False, False, x, y,
        )
        fit = self.fun([51.8, 51.9], comm)
        np.testing.assert_allclose(
            fit.val,
            [
                5.96612, 5.964306,
            ],
            rtol=1e-7,
        )
        np.testing.assert_allclose(
            fit.fwd,
            [
                5.025, 5.025,
            ],
            rtol=1e-7,
        )

class TestE01zn(IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the e01zn wrapper.
    """

class TestE01znNp(TestE01zn, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        TestE01zn.set_up(self)
        self.interp = chap.dimn_scat_shep
        self.fun = chap.dimn_scat_shep_eval

    def test_good(self):
        """
        Good call.
        """
        statecomm = rand.init_repeat(genid=1, seed=[1762543])
        d = 6
        m = 120
        x = rand.dist_uniform01(d*m, statecomm).reshape((d, m), order='F')
        f = np.empty(m)
        for i in range(m):
            f[i] = x[0, i]*x[1, i]*x[2, i]/(1. + 2.*x[3, i]*x[4, i]*x[5, i])
        comm = self.interp(x, f)
        n = 9
        xe = np.empty((d, n), order='F')
        for i in range(n):
            xe[:, i] = float(i+1)/float(n+1)
        fit = self.fun(x, f, comm.iq, comm.rq, xe)
        np.testing.assert_allclose(
            fit.q,
            [
                4.2506798853916231E-03,
                3.9743821432299637E-03,
                2.0967269970950279E-02,
                5.3601440115526637E-02,
                9.8795162153398300E-02,
                0.1525904966767604,
                0.2077399169607064,
                0.2569546704157862,
                0.2952636927108612,
            ],
        )

if __name__ == '__main__': # pragma: no cover
    unittest.main()
